import json
import logging

from google.cloud import tasks_v2 as tasks
from .project import ProjectReference


class QueueUtil:
    """
    Auxiliary class for Google Cloud Tasks API
    """

    QUEUE_HEADERS = ['X-Appengine-User-Ip', 'X-Appengine-Queuename', 'X-Appengine-Taskname',
                     'X-Appengine-Taskretrycount', 'X-Appengine-Taskexecutioncount', 'X-Appengine-Tasketa']
    QUEUE_VALID_IPS = ['10.0.0.2', '0.1.0.2']

    def __init__(self, queue_name, project=None, project_id=None, location=None):
        self.project = project if project else ProjectReference(project_id, location)
        self.client = tasks.CloudTasksClient()
        self.queue_name = queue_name
        self.queue_path = self.client.queue_path(self.project.project_id, self.project.location, self.queue_name)

    def enqueue(self, request_method, request_relative_uri, request_data=None):
        encoded_payload = (
            json.dumps(request_data).encode()
            if request_data is not None
            else None
        )

        queue = {
            'app_engine_http_request': {
                'http_method': request_method,
                'relative_uri': request_relative_uri
            }
        }

        if encoded_payload is not None:
            queue['app_engine_http_request']['body'] = encoded_payload

        self.client.create_task(self.queue_path, queue)

        # remove body to log
        if 'body' in queue['app_engine_http_request']:
            queue['app_engine_http_request'].pop('body')

        logging.info(f'Task enqeued: {queue}')

    def pause(self):
        return self.client.pause_queue(self.queue_path)

    def resume(self):
        return self.client.resume_queue(self.queue_path)

    def list_tasks(self):
        return [t for t in self.client.list_tasks(self.queue_path)]

    @classmethod
    def is_queue_request(cls, queue_name, request):
        """
        Verify if a flask request is generated by GCloud Tasks

        A push task HTTP request has special headers set by App Engine, which contain task-specific
        information your handler can use.

        If these headers are present in an external user request to your app, they are stripped and replaced.
        The sole exception is for requests from logged-in administrators of the application, who are allowed
        to set headers for testing purposes.

        Reference:
        https://cloud.google.com/appengine/docs/standard/python/taskqueue/push/creating-handlers

        :param queue_name:  Queue name
        :param request:     Flask request object
        :return:            If the request was created by a google cloud tasks job
        """
        headers = dict(t for t in request.headers if t[0] in cls.QUEUE_HEADERS)
        if set(cls.QUEUE_HEADERS) == headers.keys():
            is_a_valid_ip = headers['X-Appengine-User-Ip'] in cls.QUEUE_VALID_IPS
            return headers['X-Appengine-Queuename'] == queue_name and is_a_valid_ip

        return False
